/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.mdatool.addition;

import java.util.ArrayList;
import java.util.List;

import org.tinygroup.codegen.config.entity.EntityField;
import org.tinygroup.codegen.config.entity.EntityGroup;
import org.tinygroup.codegen.config.entity.EntityModel;
import org.tinygroup.codegen.config.entity.Operation;
import org.tinygroup.codegen.config.entity.OperationField;
import org.tinygroup.codegen.config.entity.OperationGroup;
import org.tinygroup.codegen.util.OperationObjectType;
import org.tinygroup.codegen.util.OperationType;

public final class DefaultOperation {
	/**
	 * 默认新增操作的uuid
	 */
	public static final String DEFAULT_ADD_OP_UUID = "modelopadduuid";
	/**
	 * 默认修改操作的uuid
	 */
	public static final String DEFAULT_UPDATE_OP_UUID = "modelopupdateuuid";
	/**
	 * 默认删除操作的uuid
	 */
	public static final String DEFAULT_DELETE_OP_UUID = "modelopdeleteuuid";
	/**
	 * 默认查询操作的uuid
	 */
	public static final String DEFAULT_QUERY_OP_UUID = "modelopqueryuuid";
	
	/**
	 * 默认查询查看的uuid
	 */
	public static final String DEFAULT_VIEW_OP_UUID = "modelopqueryuuid";

	private DefaultOperation(){
		
	}
	/**
	 * 为模型生成默认的增删改查操作
	 * 
	 * @param model
	 */
	public static void createDefaultOp(EntityModel model) {
		//将原有的模型操作全部清除
		model.getOperations().clear();
		model.getOperations().add(
				createOp(model, OperationType.ADD,OperationObjectType.NONE, "新增", DEFAULT_ADD_OP_UUID));
		model.getOperations().add(
				createOp(model, OperationType.UPDATE,OperationObjectType.SINGLE, "修改",
						DEFAULT_UPDATE_OP_UUID));
		model.getOperations().add(
				createOp(model, OperationType.DELETE, OperationObjectType.MUTI,"删除",
						DEFAULT_DELETE_OP_UUID));
		model.getOperations().add(
				createOp(model, OperationType.QUERY,OperationObjectType.NONE, "查询",
						DEFAULT_QUERY_OP_UUID));
		model.getOperations().add(
				createOp(model, OperationType.VIEW,OperationObjectType.SINGLE, "查看",
						DEFAULT_VIEW_OP_UUID));
	}

	/**
	 * 为模型生成默认的指定类型的操作
	 * @param model 模型
	 * @param type 操作类型
	 * @param title 操作中文名
	 * @param uuid 操作uuid
	 * @return
	 */
	private static Operation createOp(EntityModel model, OperationType type,OperationObjectType objectType,
			String title, String uuid) {
		Operation op = new Operation();
		op.setType(type.toString());
		op.setOperationObjType(objectType.toString());
		op.setConfirm(true);
		op.setEnableDelete(true);
		op.setEnableModify(true);
		op.setName(type.toString());
		op.setTitle(title);
		op.setUuid(uuid);
		List<OperationGroup> opGroups = createOpFiledGroups(model.getGroups(),
				type);
		op.getOperationGroups().addAll(opGroups);
		return op;
	}

	/**
	 * 根据模型字段分组列表生成操作字段分组列表
	 * @param groups
	 * @param type
	 * @return
	 */
	private static List<OperationGroup> createOpFiledGroups(
			List<EntityGroup> groups, OperationType type) {
		List<OperationGroup> opGroups = new ArrayList<OperationGroup>();
		for (EntityGroup group : groups) {
			opGroups.add(createOpFiledGroups(group, type));
		}
		return opGroups;
	}

	/**
	 * 根据模型字段分组生成操作字段分组
	 * @param group
	 * @param type
	 * @return
	 */
	private static OperationGroup createOpFiledGroups(EntityGroup group,
			OperationType type) {
		OperationGroup opGroup = new OperationGroup();
		opGroup.getFields().addAll(createOperationFields(group.getFields()));
		opGroup.setName(group.getName());
		opGroup.setTitle(group.getTitle());
		opGroup.setUuid(getOpGroupUUID(group, type));
		opGroup.setEnableDelete(group.getEnableDelete());
		opGroup.setEnableModify(group.getEnableModify());
		return opGroup;
	}

	/**
	 * 根据模型字段列表生成操作字段列表
	 * @param entityFields
	 * @return
	 */
	private static List<OperationField> createOperationFields(
			List<EntityField> entityFields) {
		List<OperationField> operationFields = new ArrayList<OperationField>();
		for (EntityField entityField : entityFields) {
			operationFields.add(createOperationField(entityField));
		}
		return operationFields;
	}

	/**
	 * 根据模型字段生成操作字段
	 * @param entityField
	 * @return
	 */
	private static OperationField createOperationField(EntityField entityField) {
		OperationField opField = new OperationField();
		opField.setEntityFielduuid(entityField.getUuid());
		opField.setInputMode("textMode");
		if (entityField.isPrimary()) {
			opField.setEditable(false);
			opField.setHidden(true);
		} else {
			opField.setEditable(true);
			opField.setHidden(false);
		}
		return opField;
	}

	/**
	 * 获取操作分组的uuid
	 * @param group
	 * @param type
	 * @return
	 */
	private static String getOpGroupUUID(EntityGroup group, OperationType type) {
		return String.format("%sop%s", type, group.getUuid());
	}
}
